/* x.h - Internal header file for the LibXPCE, the X PseudoColor emulation
 *       library.  Note:  This is *not* the header file that programs using
 *       the library should include.  They should include libxpce.h
 *
 * Copyright (C) 2000 Boris Gjenero
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License, version 2.1, as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You may contact Boris Gjenero at bgjenero@sympatico.ca
 */

#ifndef _LIBXPCE_INTERNAL_H_
#define _LIBXPCE_INTERNAL_H_

#include <X11/Xlib.h>
#include <X11/Xutil.h>

/* I haven't documented many things here.  If you actually want to know what
 * something does, just go into the actual .c file.
 */

/************************* TYPE DEFINITIONS **********************************/

/*** xvhandler.c ***/

#define VISINFO_COLORMAPPED 1
#define VISINFO_MONO 2
/* Uses X, but not standard windowing.  eg.  XF86DGA */
#define VISINFO_NOTXWIN 4
/* All types of alien handlers */
#define VISINFO_ALIEN VISINFO_NOTXWIN
typedef struct {
  /* Visual class */
#if defined(__cplusplus) || defined(c_plusplus)
  int c_class;
#else
  int class;
#endif
  /* Name of visual class */
  const char *name;
  /* Comment for visual class */
  const char *comment;
  /* Various flags */
  int flags;
  /* What bpp would we like? (wantbpp is optimal, >wantbpp is good) */
  int wantdepth;
  /* Setup function */
  void (*do_setup)(void);
  /* Setup function */
  void (*do_initxlat)(void);
  /* Setup function */
  void (*do_updatexlat)(unsigned char *);
  /* Cleanup function */
  void (*do_cleanup)(void);
} visinfo_t;

/************************ EXTERNAL VARIABLES *********************************/

/*** x.c ***/

extern Colormap	X_cmap;
extern Visual *X_visual;
extern XVisualInfo *X_visualinfo;
extern Window X_rootwin;
extern XImage *X_image;
extern int X_screen;
extern XColor X_colors[INPAL_SIZE];
extern Display *X_display;
extern int X_width;
extern int X_height;
extern int X_wmdetected;
extern int X_numcolors;
extern int X_u_numcolors;
extern int X_cmapin;
extern unsigned int X_bits_per_pixel;
extern unsigned int X_bytes_per_line;
extern unsigned int X_byte_order;
extern int X_setbg;
extern unsigned char *buf_graf;
extern int X_cmapupd;
extern int X_xputpixel;
extern int X_mwmapped;
extern int X_visstate;

/* What the user wants for the colormap */
#define CMAPPREF_UNSPEC 0
#define CMAPPREF_PRIVATE 1
#define CMAPPREF_DEFAULT 2 
extern int X_cmappref; 

#ifdef COLORMAP_THEFT
/* We want to grab the colormap */
#define GRABCMAP_NO 0
#define GRABCMAP_ICCCM 1
#define GRABCMAP_ROOT 2
extern int X_grabcmap;
#endif 

/* Full screen display mode */
extern int X_fsmode;
/* Window with size and position hints */
#define FSMODE_NORMAL 0
/* Override-redirect window */
#define FSMODE_OR 1
#ifdef XF86DGA
/* XFree-86 DGA */
#define FSMODE_DGA 2
#endif

extern int X_colormapped;
extern int X_privatecmap;

/* Display mode specifications bitmask */
#define DMODE_UNSPECIFIED 0
#define DMODE_XSET 1
#define DMODE_YSET 2
#define DMODE_FULLSCREEN 4
#define DMODE_ROOT 8
extern int X_dmode;

/* If this is non-zero, XAllocColor will be used instead of our much
   faster routines */
#define XALLOCCOLOR_USER 1
#define XALLOCCOLOR_PROGRAM 2 
extern int X_xalloccolor;

/*** xxlat.c ***/

/* Translation requirements */
/* Need translation due to fixed colormap */
#define NEEDXLAT_FIXCMAP 1
/* Need translation due colormap entries we are using */
#define NEEDXLAT_CMAP 2
/* Need translation due to bytes per line */
#define NEEDXLAT_BPL 4
#define NEEDXLAT_ON_CMAPCHG NEEDXLAT_FIXCMAP
#define NEEDXLAT_USE_XLATTAB (NEEDXLAT_FIXCMAP | NEEDXLAT_CMAP)
extern int X_needxlat;
extern unsigned long int X_xlattab1[INPAL_SIZE];
#define X_xlattab X_xlattab1

/* Destination for translation */
unsigned char *X_xlatdest;

extern void (*X_sxlatfunc)(unsigned char *,  /* Destination */ 
			   unsigned int, unsigned int,  /* x, y */
			   unsigned int, unsigned int   /* width, height */
			   );

/*** xvhandler.c ***/

#ifdef XF86DGA
extern int X_dga;
extern unsigned char *X_dga_addr;
extern int X_dblbuf;
#endif

extern unsigned long X_red_mask, X_green_mask, X_blue_mask;

extern XColor *X_cmapinfo;

extern int X_visdef_handler;
extern int X_visdef_fshand;
extern int X_visdef_id;


/* The type of request the user has made */
#define VISDEF_UNDEFINED 0
#define VISDEF_DEFAULT 1
#define VISDEF_ID 2
#define VISDEF_HANDLER 4
#define VISDEF_FSHAND 8
extern int X_visdeftype;

extern visinfo_t *X_vhandler;

/*********************** FUNCTION PROTOTYPES *********************************/

/*** x.c ***/

void x_Error(const char *);
void x_GrabServer(void);
void x_UngrabServer(void);
int x_GetPixmapFormat(int depth);

/*** xxlat.c ***/

void x_Endiannize(unsigned long int *start,
		  unsigned int spacing,
		  unsigned int n);
void x_PrecalcXlatShift(void);
void x_SXlatNull(unsigned char *d, 
		 unsigned int x, unsigned int y, 
		 unsigned int w, unsigned int h);
void x_SelectXlatFunc(void);
void x_XlatBuffer();

/*** vhandler.c ***/

void x_PickVisual(void);

#ifdef XF86DGA
void x_InitXF86DGA(void);
void x_CleanupXF86DGA(void);
void x_XF86DGASwapBuf(void);
#endif

void x_ListVis(void);
int x_SetWantedVis(char *s, unsigned int mask);

#endif

