#include <pthread.h>
#include <stdlib.h>
#include <stdio.h>

#include "gUnixBase.h"

#ifdef HAVE_CONFIG_H
#include <autoconf.h>
#else
#define VERSION ""
#endif

pthread_mutex_t sample_mutex;
int once_inited = 0;

static void gf_configure(void);
static void gf_about(void);
static void gf_init(void);
static void gf_cleanup(void);
static void gf_playback_start(void);
static void gf_playback_stop(void);
static void gf_render_pcm(signed short int data[2][512]);
static void bumpscope_about(void);

/* Just WHY do the XMMS headers depend on GTK headers? */
typedef struct _VisPlugin
{
  void *handle;   /* Filled in by xmms */
  char *filename; /* Filled in by xmms */
  int xmms_session; /* The session ID for attaching to the control socket
		     */
  char *description; /* The description that is shown in the preferences box */
  int num_pcm_chs_wanted; /* Numbers of PCM channels wanted in the call to render_pcm */
  int num_freq_chs_wanted; /* Numbers of freq channels wanted in the call
			      to render_freq */
  void (*init)(void); /* Called when the plugin is enabled */
  void (*cleanup)(void); /* Called when the plugin is disabled */
  void (*about)(void); /* Show the about box */
  void (*configure)(void); /* Show the configure box */
  void (*disable_plugin)(struct _VisPlugin *); /* Call this with a pointer to your plugin to disable the plugin */
  void (*playback_start)(void); /* Called when playback starts */
  void (*playback_stop)(void); /* Called when playback stops */
  void (*render_pcm)(signed short int pcm_data[2][512]); /* Render the PCM data, don't do anything time consuming in here */
  void (*render_freq)(signed short int freq_data[2][256]);  /* don't do anything time consuming in here */
} VisPlugin;


VisPlugin gforce_vp = {
  NULL,              /* XMMS fills:  Handle */
  NULL,              /* XMMS fills:  Filename */
  0,                 /* XMMS fills:  Session ID */
  "G-Force "VERSION, /* Description */
  1,                 /* Number of PCM channels wanted */
  0,                 /* Number of freq channels wanted */
  gf_init,           /* init */
  gf_cleanup,        /* cleanup */
  gf_about,          /* about */
  gf_configure,      /* configure */
  NULL,              /* XMMS fills: disable_plugin */
  gf_playback_start, /* playback_start */
  gf_playback_stop,  /* playback_stop */
  gf_render_pcm,     /* render_pcm */
  NULL               /* render_freq (we don't use freq channels) */
};

#define NUMSAMPLES 200
#define SAMPSKIP 1
unsigned short int sbuf[NUMSAMPLES];

static pthread_t render_thread;

int quitflag;

/* This is called for xpce_HandleEvents() */
void xpceC_QuitRequest(void) {
  /* This effectively calls gf_cleanup.  That should cause deadlock, but
   * I guess that call comes from a different thread, so everything works.
   */
  gforce_vp.disable_plugin(&gforce_vp);  
}

VisPlugin *get_vplugin_info(void) {
  return &gforce_vp;
}

static void *gf_render_thread(void *arg) {
  /* This is the render thread.  It just does rendering until
   * it's told to quit.  Then it does cleanup.
   */
  while(1) {
    /* Handle X events. */
    xpce_HandleEvents();

    /* An X event could have set quitflag */
    if (quitflag) break;

    /* Send samples to G-Force.  The mutex is used so we never draw
     * part-new/part-old data.  This is done separately from Render so 
     * we don't block the other thread for long.
     */
    pthread_mutex_lock(&sample_mutex);
    UploadSamples(sbuf, NUMSAMPLES);
    pthread_mutex_unlock(&sample_mutex);

    /* Actually render using those samples */
    Render();
  }
  
  /* Cleanup starts here.  It's done in the render thread so it works
   * properly regardless of whether we were disabled with a
   * WM_DELETE_WINDOW message or from XMMS
   */
  xpce_QuitNotify();

  /* ::KillTimer( gGF_Win, 5556 ); */

  Quit();

  pthread_exit(NULL);
}

static void gf_init(void)
{
  /* Reset quitflag if we had run and quit before */
  quitflag = 0;

  if (!once_inited) {
    pthread_mutex_init(&sample_mutex, NULL);

    once_inited = 1;
  }

  xpce_SetWindowCaption("G-Force");

  Init();

  /* ::SetTimer( gGF_Win, 5556, 10, NULL ); */

  /* CPU intensive stuff is done in a separate thread so the XMMS thread
   * that calls the plugin isn't affected
   */
  pthread_create(&render_thread, NULL, gf_render_thread, NULL);
}

static void gf_cleanup(void)
{
  /* Make sure the render thread will quit */
  quitflag = 1;

  /* At this point the render thread does cleanup and we wait for it */
  pthread_join(render_thread, NULL);

  /* All done */
}

static void gf_playback_start(void) {
}

static void gf_playback_stop(void) {
}

static void gf_render_pcm(signed short int data[2][512])
{
  /*  static int smin = 0, smax = 0; */
  int i, j;

  /* Lock mutex so we never draw part-old/part-new */
  pthread_mutex_lock(&sample_mutex);

  /* Store samples in a buffer that the render thread will eventually get to */
  j = 0;
  for (i = 0; i < NUMSAMPLES; i++) {
 #if 0
    /* Interesting.  The samples are never even close to +/-32767 */
    if (data[0][j] < smin) {
      smin = data[0][j];
      fprintf(stderr, "min = %i\n", data[0][j]);
    } else if (data[0][j] > smax) {
      smax = data[0][j];
      fprintf(stderr, "MAX = %i\n", data[0][j]);
    }
#endif

    sbuf[i] = data[0][j];
#ifdef SAMPSKIP
    j += 1 + SAMPSKIP;
#else
    j++;
#endif
  }

  pthread_mutex_unlock(&sample_mutex); 
}

static void gf_about(void) {
  fprintf(stderr, "gf_about (unimplemented)\n");
}

static void gf_configure(void) {
  fprintf(stderr, "gf_configure (unimplemented, start the plugin and "
	  "press ?\n");
}




