#include <gtk/gtk.h>
#include <gdk/gdk.h>

#include <string.h>
#include <stdlib.h>

#include "bl.xpm"
#include "br.xpm"
#include "tl.xpm"
#include "tr.xpm"
#include "tail_ul.xpm"
#include "tail_ur.xpm"
#include "tail_dl.xpm"
#include "tail_dr.xpm"

#include "speech-bubble.h"

#define FONT "-adobe-helvetica-bold-r-normal--*-120-*-*-*-*-iso8859-*"

static int have_inited = 0;
static GtkStyle *style;
static GdkFont *font;
static guint line_height;

void
vigor_speech_bubble_init (void)
{
  style = gtk_widget_get_default_style();
  font = gdk_font_load(FONT);
  line_height = font->ascent + font->descent;
  
  have_inited = 1;
}

static void *
vigor_speech_bubble (const char* to_render, int xpos, int ypos,
		     int vigor_is_above, int vigor_is_left)
{
  GtkWidget *window;
  GtkWidget *pixmap;
  GdkGC *pixmap_gc, *mask_gc;
  guint text_height, text_width;
  guint baseline;
  char *to_rend, *line;
  GdkPixmap *tl_pixmap, *tr_pixmap, *bl_pixmap, *br_pixmap, *tail_pixmap;
  GdkBitmap *tl_mask, *tr_mask, *bl_mask, *br_mask, *tail_mask;
  GdkPixmap *wind_pixmap;
  GdkBitmap *wind_mask;

  /* Create the main window, and attach delete_event signal to terminate
   * the application.  Note that the main window will not have a titlebar
   * since we're making it a popup. */
  window = gtk_window_new (GTK_WINDOW_POPUP);
  /* gtk_signal_connect (GTK_OBJECT (window), "delete_event",
   * GTK_SIGNAL_FUNC (close_application), NULL); */
  gtk_widget_show (window);

  /* I can't find any way to use GTK to do this stuff (because of
   * the complexity of the shape), so I create a pixmap with the
   * content using GDK and then fill it in. */
  /* First, we figure out the width and height of the string. */
  /* GDK doesn't provide us with any way to access the font's
   * FONT_ASCENT property; it only provides the maximum ascent
   * property.  So we have to go with that. */
  to_rend = strdup(to_render);
  text_width = text_height = 0;
  /* FIXME I shouldn't use strtok in a plugin, and it doesn't let me
   * have blank lines.  Copy in BSD's strsep instead. */
  for (line = strtok(to_rend, "\n"); line; line = strtok(NULL, "\n")) {
    gint line_width;
    text_height += line_height;
    /* FIXME This actually takes into account intercharacter spacing
     * at the end, which I don't want.  Should use string_extents. */
    line_width = gdk_string_width(font, line);
    if (line_width > text_width)
      text_width = line_width;
  }
  free(to_rend);
    
  tail_pixmap = gdk_pixmap_create_from_xpm_d(window->window, &tail_mask,
					     &style->bg[GTK_STATE_NORMAL],
					     vigor_is_above && vigor_is_left ? tail_ul_xpm :
					     vigor_is_above && !vigor_is_left ? tail_ur_xpm :
					     !vigor_is_above && vigor_is_left ? tail_dl_xpm :
					     !vigor_is_above && !vigor_is_left ? tail_dr_xpm :
					     NULL);
  tl_pixmap = gdk_pixmap_create_from_xpm_d(window->window, &tl_mask,
					   &style->bg[GTK_STATE_NORMAL],
					   tl_xpm);
  tr_pixmap = gdk_pixmap_create_from_xpm_d(window->window, &tr_mask,
					   &style->bg[GTK_STATE_NORMAL],
					   tr_xpm);
  bl_pixmap = gdk_pixmap_create_from_xpm_d(window->window, &bl_mask,
					   &style->bg[GTK_STATE_NORMAL],
					   bl_xpm);
  br_pixmap = gdk_pixmap_create_from_xpm_d(window->window, &br_mask,
					   &style->bg[GTK_STATE_NORMAL],
					   br_xpm);

  /* While we do all of this, we also set the mask at the same
   * time. */

  /* Create the final pixmap. */
  /* I have no idea why GDK doesn't provide a way to get metrics
   * from pixmaps. */
#define ROUND_RADIUS 20
  /* The defined tail height is actually one shorter than the pixmap
   * so that the pixmap's white line at the bottom extends into
   * the white rect's place. */
#define TAIL_HEIGHT 19
  wind_pixmap = gdk_pixmap_new (window->window,
				text_width + ROUND_RADIUS * 2,
				text_height + ROUND_RADIUS * 2 + TAIL_HEIGHT,
				-1);
  pixmap_gc = gdk_gc_new(wind_pixmap);
  wind_mask = gdk_pixmap_new (window->window,
			      text_width + ROUND_RADIUS * 2,
			      text_height + ROUND_RADIUS * 2 + TAIL_HEIGHT,
			      1);
  mask_gc = gdk_gc_new(wind_mask);

  /* Draw in the broad regions. */
  gdk_gc_set_function (pixmap_gc, GDK_SET);
  gdk_draw_rectangle (wind_pixmap, pixmap_gc, TRUE, 0,0, 
		      text_width + ROUND_RADIUS * 2,
		      text_height + ROUND_RADIUS * 2 + TAIL_HEIGHT);
  gdk_gc_set_function (pixmap_gc, GDK_CLEAR);
  gdk_draw_rectangle (wind_pixmap, pixmap_gc, FALSE,
		      0,(vigor_is_above ? TAIL_HEIGHT : 0),
		      text_width + ROUND_RADIUS * 2 - 1,
		      text_height + ROUND_RADIUS * 2 - 1);
  gdk_gc_set_function (pixmap_gc, GDK_COPY);
  gdk_gc_set_function (mask_gc, GDK_SET);
  gdk_draw_rectangle (wind_mask, mask_gc, TRUE, 
		      0,0,
		      text_width + ROUND_RADIUS * 2,
		      text_height + ROUND_RADIUS * 2 + TAIL_HEIGHT);
  gdk_gc_set_function (mask_gc, GDK_CLEAR);
  gdk_draw_rectangle (wind_mask, mask_gc, TRUE,
		      0,(vigor_is_above ? 0 : (text_height + ROUND_RADIUS * 2)), 
		      text_width + ROUND_RADIUS * 2,TAIL_HEIGHT);
  gdk_gc_set_function (mask_gc, GDK_COPY);

  /* Draw in the text. */
  baseline = (vigor_is_above ? TAIL_HEIGHT : 0) + ROUND_RADIUS + font->ascent;
  /* FIXME strsep */
  to_rend = strdup(to_render);
  for (line = strtok(to_rend, "\n"); line; line = strtok(NULL, "\n")) {
    gdk_draw_string (wind_pixmap, font, pixmap_gc,
		     ROUND_RADIUS, baseline,
		     line);
    baseline += line_height;
  }
  free(to_rend);

  /* Draw in the decorations. */
  gdk_draw_pixmap(wind_pixmap, pixmap_gc, tail_pixmap, 0,0,
		  (text_width>>1) + ROUND_RADIUS,
		  (vigor_is_above ? 0 : (text_height + 2*ROUND_RADIUS - 1)),
		  -1,-1);
  gdk_draw_pixmap(wind_pixmap, pixmap_gc, tl_pixmap, 0,0,
		  0,(vigor_is_above ? TAIL_HEIGHT : 0),
		  -1,-1);
  gdk_draw_pixmap(wind_pixmap, pixmap_gc, tr_pixmap, 0,0,
		  text_width + ROUND_RADIUS, (vigor_is_above ? TAIL_HEIGHT : 0),
		  -1,-1);
  gdk_draw_pixmap(wind_pixmap, pixmap_gc, bl_pixmap, 0,0,
		  0, text_height + ROUND_RADIUS + (vigor_is_above ? TAIL_HEIGHT : 0),
		  -1,-1);
  gdk_draw_pixmap(wind_pixmap, pixmap_gc, br_pixmap, 0,0,
		  text_width + ROUND_RADIUS,
		  text_height + ROUND_RADIUS + (vigor_is_above ? TAIL_HEIGHT : 0),
		  -1,-1);

  gdk_draw_pixmap(wind_mask, mask_gc, tail_mask, 0,0,
		  (text_width>>1) + ROUND_RADIUS,
		  (vigor_is_above ? 0 : (text_height + 2*ROUND_RADIUS)),
		  -1,-1);
  gdk_draw_pixmap(wind_mask, mask_gc, tl_mask, 0,0,
		  0,(vigor_is_above ? TAIL_HEIGHT : 0),
		  -1,-1);
  gdk_draw_pixmap(wind_mask, mask_gc, tr_mask, 0,0,
		  text_width + ROUND_RADIUS, (vigor_is_above ? TAIL_HEIGHT : 0),
		  -1,-1);
  gdk_draw_pixmap(wind_mask, mask_gc, bl_mask, 0,0,
		  0, text_height + ROUND_RADIUS + (vigor_is_above ? TAIL_HEIGHT : 0),
		  -1,-1);
  gdk_draw_pixmap(wind_mask, mask_gc, br_mask, 0,0,
		  text_width + ROUND_RADIUS,
		  text_height + ROUND_RADIUS + (vigor_is_above ? TAIL_HEIGHT : 0),
		  -1,-1);

  /* Now for the pixmap and the pixmap widget */
  pixmap = gtk_pixmap_new (wind_pixmap, NULL);
  gtk_widget_show (pixmap);

  /* Finish up the window */
  gtk_widget_shape_combine_mask (window, wind_mask, 0, 0);
  gtk_container_add (GTK_CONTAINER(window), pixmap);
  gtk_widget_set_uposition (window, xpos, ypos);

  /* Clean up everything we're done using. */
  /* I think we have to keep the pixmap. */
  /* The GTK objects that have been parented are fine.  They'll get
   * derefd later. */
#if 0
  gdk_gc_unref(pixmap_gc);
  gdk_gc_unref(mask_gc);
  gdk_font_unref(font);
  gtk_style_unref(style);
  gdk_pixmap_unref(tl_pixmap);
  gdk_pixmap_unref(tr_pixmap);
  gdk_pixmap_unref(bl_pixmap);
  gdk_pixmap_unref(br_pixmap);
  gdk_pixmap_unref(tail_pixmap);
  gdk_bitmap_unref(tl_mask);
  gdk_bitmap_unref(tr_mask);
  gdk_bitmap_unref(bl_mask);
  gdk_bitmap_unref(br_mask);
  gdk_bitmap_unref(tail_mask);
  gdk_pixmap_unref(wind_pixmap);
  gdk_bitmap_unref(wind_mask);
#endif
 
  gtk_widget_ref (window);
  gtk_widget_show (window);
  return(window);
}

static const char * sayings[] = {
  "Are you sure you wish to\nlisten to the chorus?",
  "Are you sure you wish to\nlisten to this verse?",
  "Are you sure you wish to\nlisten to the bridge?",
  "Are you sure you wish to\nlisten to this song?",
  "You appear to be listening to a song!\nWould you like help with that?",
  "You appear to be listening to a song!\nWould you like help with that?",
  "You appear to be listening to a song!\nWould you like help with that?",
  "You appear to be listening to a song!\nWould you like help with that?",
  "You appear to be listening to a song!\nWould you like help with that?",
  "Are you sure you wish to\nhave a spontanious pohn fahr?",
  "Don't cry --- it won't help.",
  "Don't try this at home, kids!",
  "Or perhaps you'd prefer Russian Routlette, after all?",
  "You'll understand when you're older, dear.",
  "Want some rye?  'Course ya do!",
  "Your blood pressure just went up a point.",
  "Your blood pressure just went up a point.",
  "Your blood pressure just went up a point.",
  "Please be my friend...",
  "I see you're writing a suicide note!\nWould you like help with this task?",
  "Vigor: A compelling argument for pencil and paper.",
  "Vigor: An inspiring example of form following function... to Hell.",
  "Vigor: Because Hell was full.",
  "Vigor is freely redistributable;\nvoid where prohibited by law.",
  "If SIGINT doesn't work, try a tranquilizer.",
  "It's all fun and games, until somebody tries to edit a file.",
  "Lovecraft was an optimist.",
  "Vigor: More than just a pretty face!",
  "Vigor: The only text editor known to get indigestion.",
  "Vigor: Where editing text is like playing Paganini on a glass harmonica.",
  "Vigor: All the problems and twice the bugs.",
  "Vigor: Dissatisfaction guaranteed.",
  "Vigor: Don't get frustrated without it.",
  "Vigor: Even not doing anything would have been better than nothing.",
  "Vigor: Even your dog won't like it.",
  "Vigor: Form follows malfunction.",
  "Vigor: It could be worse, but it'll take time.",
  "Vigor: It could happen to you.",
  "Vigor: Let it get in *your* way.",
  "Vigor: Live the nightmare.",
  "Vigor: Putting new limits on productivity.",
  "Vigor: Some voids are better left unfilled.",
  "Vigor: The problem for your problem.",
  "Vi: It was hard to write; it should be hard to use.",
  "VI VI VI: The editor of the beast",
  "Vigor: Putting the ass in assist",
  "X Windows: A mistake carried out to perfection.",
  "X Windows: A terminal disease."
};

void *
vigor_speech_bubble_create (int saying_number, int vigor_xpos, int vigor_ypos,
			    int screen_width, int screen_height)
{
  int saying_count = sizeof(sayings) / sizeof(sayings[0]);
  int vigor_on_left = vigor_xpos < (screen_width >> 1);
  int vigor_on_top = vigor_ypos < (screen_height >> 1);
  saying_number = saying_number % saying_count;
  /* XXX There's nothing sacred about these positioning adjustments,
   * and it would be good to do something more intelligent. */
  return vigor_speech_bubble (sayings[saying_number], 
			      vigor_on_left ? vigor_xpos : (vigor_xpos - 75),
			      vigor_on_top ? (vigor_ypos + 150) : (vigor_ypos - 150),
			      vigor_on_top, vigor_on_left);
}

void
vigor_speech_bubble_event (void)
{
  //gtk_main_iteration_do(FALSE);
}

void
vigor_speech_bubble_destroy (GtkWidget *window)
{
  gtk_widget_hide_all (window);
  gtk_widget_unref (window);
  //gtk_main_iteration_do(FALSE);
}

void
vigor_speech_bubble_shutdown (void)
{
  /* XXX How am I supposed to release GTK's global resources? */
}
